﻿using System;
using System.Text;
using com.epson.bank.driver;

namespace TM_S9000Sample
{
    class CPrint
    {
        private const uint TRANS_NO_SPECIFIED = 0x40000000;
        private CProperty_1 m_objProperty = null;

        public CPrint()
        {
	        m_objProperty = new CProperty_1();
        }

        public ErrorCode Init(CProperty_1 objProperty)
        {
	        ErrorCode errResult = ErrorCode.SUCCESS;

            m_objProperty = objProperty;

	        return errResult;
        }

        // Print on roll paper
        public ErrorCode PrintRollpaper()
        {
            // S9000 only
	        ErrorCode errResult = ErrorCode.SUCCESS;

            // Set printing unit to roll paper
            errResult = m_objProperty.GetMfDevice().SetPrintStation(PrintingStation.MF_ST_ROLLPAPER);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            // Start print data buffering
            errResult = m_objProperty.GetMfDevice().BufferedPrint(PrintBuffer.MF_PRT_BUFFERING);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
            }

            // Set image data for printing
            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_CENTER);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SetPrintSize(65, 15);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintImage("Logo.bmp");
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set printing text
            MFDeviceFont mfDecorate = new MFDeviceFont();
            string szRollPaper = null;
            StringBuilder sbRollPaper = new StringBuilder();
            DateTime stDateTime;

            mfDecorate.Font = FontType.MF_PRINT_FONT_B;
            mfDecorate.Size = FontScale.MF_PRINT_FONT_W1_H1;
            mfDecorate.Underline = UnderlineType.MF_PRINT_NO_UNDERLINE;
            mfDecorate.Color = PrintColor.MF_PRINT_BLACK;

            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_RIGHT);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
            }

            sbRollPaper.Append("\r\n\r\n600 Grant Street Pittsburgh, PA 15219\r\n");
            sbRollPaper.Append("Phone: (555)555-5555\r\n\r\n\r\n");
            szRollPaper = sbRollPaper.ToString();
            sbRollPaper.Length = 0;

            errResult = m_objProperty.GetMfDevice().PrintText(szRollPaper, mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }


            mfDecorate.Font = FontType.MF_PRINT_FONT_A;
            stDateTime = DateTime.Now;

            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_LEFT);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            sbRollPaper.Append("Station S1             Teller 0001 Seq# 02\r\n");
            sbRollPaper.Append(stDateTime.Day.ToString("00") + "/" + stDateTime.Month.ToString("00")
                                + "/" + stDateTime.Year.ToString("00") + "                         "
                                + stDateTime.Hour.ToString("00") + ":" + stDateTime.Minute.ToString("00") + ":"
                                + stDateTime.Second.ToString("00") + "\r\n\r\n");
            szRollPaper = sbRollPaper.ToString();
            sbRollPaper.Length = 0;

            errResult = m_objProperty.GetMfDevice().PrintText(szRollPaper, mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }


            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_CENTER);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintText("Deposit Transaction Receipt\r\n\r\n", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }


            mfDecorate.Underline = UnderlineType.MF_PRINT_UNDERLINE_1;

            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_LEFT);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintText("Deposit Ticket                   Amount($)\r\n", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }


            mfDecorate.Underline = UnderlineType.MF_PRINT_NO_UNDERLINE;

            errResult = m_objProperty.GetMfDevice().PrintText("Account #1234567890               1,000.00\r\n\r\n", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }


            mfDecorate.Underline = UnderlineType.MF_PRINT_UNDERLINE_1;

            errResult = m_objProperty.GetMfDevice().PrintText("Demosited Items                  Amount($)\r\n", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }


            mfDecorate.Underline = UnderlineType.MF_PRINT_NO_UNDERLINE;

            sbRollPaper.Append("Personal Check\r\n");
            sbRollPaper.Append("1234567890-1541                   1,000.00\r\n\r\n");
            szRollPaper = sbRollPaper.ToString();
            sbRollPaper.Length = 0;

            errResult = m_objProperty.GetMfDevice().PrintText(szRollPaper, mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_RIGHT);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintText("---------\r\n", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_LEFT);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintText("Total:                            1,000.0\r\n", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_CENTER);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintText("Successful Deposit Transaction.\n\r\r\n", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set printing barcode
            errResult = m_objProperty.GetMfDevice().SetPrintAlignment(Alignment.MF_PRINT_ALIGNMENT_CENTER);	
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().SetPrintSize(30, 15);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            byte[] byBarcode = Encoding.UTF8.GetBytes("123456789");

            errResult = m_objProperty.GetMfDevice().PrintBarCode(byBarcode, Symbol.CODE_CODE39, TextPosition.HRI_BELOW);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            byBarcode = Encoding.UTF8.GetBytes("EXCEED YOUR VISION EXCEED YOUR VISION EXCEED YOUR VISION EXCEED YOUR VISION EXCEED YOUR VISION");
            errResult = m_objProperty.GetMfDevice().PrintBarCode(byBarcode, Symbol.CODE_PDF417, TextPosition.HRI_NONE);
            if(errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            //Auto cut
            errResult = m_objProperty.GetMfDevice().AutoCutRollPaper(AutoCutType.AUTOCUT_FEEDANDCUT);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // print buffered printing data
            errResult = m_objProperty.GetMfDevice().BufferedPrint(PrintBuffer.MF_PRT_EXEC);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

	        return errResult;

        }

        // Print on cut sheet paper
        public ErrorCode PrintReceipt()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;
            int nLoadNum = 0;
            StringBuilder sbPath = new StringBuilder();
            string szPath = null;

            // Set printing unit to cut sheet paper
            errResult = m_objProperty.GetMfDevice().SetPrintStation(PrintingStation.MF_ST_FEEDER);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Load print information file for template printing
            sbPath.Append(System.IO.Directory.GetCurrentDirectory());
            sbPath.Append("\\FormatSettingStep5.ini");
            szPath = sbPath.ToString();

            errResult = m_objProperty.GetMfDevice().LoadTemplatePrintArea(szPath, out nLoadNum);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SetPrintCutSheetSettings(1, ScanEnable.BI_PRINTCUTSHEET_NOSCAN);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfBase(), FunctionType.MF_GET_BASE_DEFAULT);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfBase(), FunctionType.MF_SET_BASE_PARAM);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintCutSheet(null, FunctionType.MF_EXEC);

	        return errResult;

        }

        // Set contents of cut sheet paper printing
        public ErrorCode PrintReceiptFunction(int nTransactionNumber)
        {
	        ErrorCode errResult = ErrorCode.SUCCESS;

            // Start buffering of print data for template printing
            errResult = m_objProperty.GetMfDevice().TemplatePrint(TemplatePrintMode.TEMPLATEPRINT_BUFFERING);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            // Set image data for printing
            errResult = m_objProperty.GetMfDevice().SetTemplatePrintArea(AreaSelectMode.SELECTPRINTAREA_AREANAME, "ReceiptLogo", null, TRANS_NO_SPECIFIED);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintImage("ReceiptImage.bmp");
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            // Set print text
            errResult = m_objProperty.GetMfDevice().SetTemplatePrintArea(AreaSelectMode.SELECTPRINTAREA_AREANAME, "ReceiptText", null, TRANS_NO_SPECIFIED);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
            }

            DateTime        stDateTime;
            MFDeviceFont    mfDecorate  = new MFDeviceFont();
            string          szReceipt   = null;
            StringBuilder   sbReceipt   = new StringBuilder();
            string[]        szWeek      = new string[]{"", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};

            stDateTime = DateTime.Now;
            sbReceipt.Append(stDateTime.Day.ToString("00") + "/" + stDateTime.Month.ToString("00") + "/"
                                + stDateTime.Year.ToString("0000") + " " + szWeek[(int)stDateTime.DayOfWeek] + " "
                                + stDateTime.Hour.ToString("00") + ":" + stDateTime.Minute.ToString("00") + ":"
                                + stDateTime.Second.ToString("00") + ":" + stDateTime.Millisecond.ToString() + "\r\n");
            sbReceipt.Append("nTran #" + nTransactionNumber.ToString("00000") + "\r\n");
            sbReceipt.Append("nEntity NCS CC 9999999 Tlr 99999\r\n");
            sbReceipt.Append("Acct Type SAV\r\n");
            sbReceipt.Append("Available Balance\r\t2000.00\r\n");
            szReceipt = sbReceipt.ToString();

            errResult = m_objProperty.GetMfDevice().PrintText(szReceipt, mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Print buffered printing data
            errResult = m_objProperty.GetMfDevice().TemplatePrint(TemplatePrintMode.TEMPLATEPRINT_EXEC);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

	        return errResult;

        }

        // Set contents of cashiers check printing
        public ErrorCode SetCashierscheckPrintSetting()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;
            int nLoadNum = 0;
            StringBuilder sbPath = new StringBuilder();
            string szPath = null;

            // Set printing unit to cut sheet paper
            errResult = m_objProperty.GetMfDevice().SetPrintStation(PrintingStation.MF_ST_FEEDER);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Load print information file for template printing
            sbPath.Append(System.IO.Directory.GetCurrentDirectory());
            sbPath.Append("\\FormatSettingStep5.ini");
            szPath = sbPath.ToString();

            errResult = m_objProperty.GetMfDevice().LoadTemplatePrintArea(szPath, out nLoadNum);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SetPrintCutSheetSettings(m_objProperty.GetPrintingNumber(), m_objProperty.GetAfterScanning());
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            return errResult;
        }

        // Set MF_SCAN structure for cashiers check printing
        public ErrorCode SetCashierscheckScanSetting()
        {
            ErrorCode errResult = ErrorCode.SUCCESS;

            errResult = m_objProperty.GetMfDevice().SCNSelectScanUnit(ScanUnit.EPS_BI_SCN_UNIT_CHECKPAPER);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SCNSetImageTypeOption(ImageTypeOption.EPS_BI_SCN_OPTION_COLOR);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().SCNSetImageTypeOption(ImageTypeOption.EPS_BI_SCN_OPTION_COLOR);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Set scanning setting for printed side
            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfScanFront(), FunctionType.MF_GET_SCAN_PRINTAREA_DEFAULT);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfScanFront(), FunctionType.MF_SET_SCAN_PRINTAREA_PARAM);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            // Set scanning setting for reverse side
            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfScanBack(), FunctionType.MF_GET_SCAN_NOPRINTAREA_DEFAULT);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfScanBack(), FunctionType.MF_SET_SCAN_NOPRINTAREA_PARAM);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

	        return errResult;
        }

        // Set MF_BASE structure for cashiers check printing
        public ErrorCode SetCashierscheckBaseSetting()
        {
	        ErrorCode errResult = ErrorCode.SUCCESS;

            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfBase(), FunctionType.MF_GET_BASE_DEFAULT);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintCutSheet(m_objProperty.GetMfBase(), FunctionType.MF_SET_BASE_PARAM);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

	        return errResult;
        }

        // Perform cashiers check printing
        public ErrorCode PrintCashierscheck()
        {
	        ErrorCode errResult = ErrorCode.SUCCESS;

	        errResult = m_objProperty.GetMfDevice().PrintCutSheet(null, FunctionType.MF_EXEC);

	        return errResult;
        }

        // Set contents of cashiers check printing
        public ErrorCode PrintCashierscheckFunction()
        {
	        ErrorCode errResult = ErrorCode.SUCCESS;

            // Start buffering of print data for template printing
            errResult = m_objProperty.GetMfDevice().TemplatePrint(TemplatePrintMode.TEMPLATEPRINT_BUFFERING);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            // Set image data for printing
            errResult = m_objProperty.GetMfDevice().SetTemplatePrintArea(AreaSelectMode.SELECTPRINTAREA_AREANAME, "CashiersSignature", null, TRANS_NO_SPECIFIED);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            errResult = m_objProperty.GetMfDevice().PrintImage("CashiresSign.bmp");
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

            // Set print text
            DateTime stDateTime;
            MFDeviceFont mfDecorate         = new MFDeviceFont();
            string szCashierscheck          = null;
            StringBuilder sbCashierscheck   = new StringBuilder();
            string[] szMonth                = new string[] { "", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

            /*Date*/
            stDateTime              = DateTime.Now;
            mfDecorate.Underline    = UnderlineType.MF_PRINT_NO_UNDERLINE;
            mfDecorate.Font         = FontType.MF_PRINT_FONT_A;
            mfDecorate.Size         = FontScale.MF_PRINT_FONT_W1_H1;

            errResult = m_objProperty.GetMfDevice().SetTemplatePrintArea(AreaSelectMode.SELECTPRINTAREA_AREANAME, "CashiersDate", null, TRANS_NO_SPECIFIED);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            sbCashierscheck.Append(szMonth[stDateTime.Month] + " " + stDateTime.Day.ToString("00") 
                                                            + "," + stDateTime.Year.ToString("0000"));
            szCashierscheck = sbCashierscheck.ToString();
            sbCashierscheck.Length = 0;

            errResult = m_objProperty.GetMfDevice().PrintText(szCashierscheck, mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            /*Name*/
            mfDecorate.Size = FontScale.MF_PRINT_FONT_W2_H1;

            errResult = m_objProperty.GetMfDevice().SetTemplatePrintArea(AreaSelectMode.SELECTPRINTAREA_AREANAME, "CashiersName", null, TRANS_NO_SPECIFIED);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintText("Taro Nanashi", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            /*OrderNum*/
            errResult = m_objProperty.GetMfDevice().SetTemplatePrintArea(AreaSelectMode.SELECTPRINTAREA_AREANAME, "CashiersOrderString", null, TRANS_NO_SPECIFIED);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            errResult = m_objProperty.GetMfDevice().PrintText("Two thousand five hundred and 00 Cents", mfDecorate);
            if (errResult != ErrorCode.SUCCESS)
            {
                return errResult;
            }

            // Print buffered printing data
            errResult = m_objProperty.GetMfDevice().TemplatePrint(TemplatePrintMode.TEMPLATEPRINT_EXEC);
	        if(errResult != ErrorCode.SUCCESS)
	        {
		        return errResult;
	        }

	        return errResult;
        }

		// Check if the roll paper is supported as print station.
		public bool IsRollPaperSupported()
		{
			ErrorCode errResult = ErrorCode.SUCCESS;
			errResult = m_objProperty.GetMfDevice().SetPrintStation(PrintingStation.MF_ST_ROLLPAPER);
			if (errResult == ErrorCode.SUCCESS)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	}
}
