#include "stdafx.h"
#include "Endorse.h"
#include <shlwapi.h>

CEndorse::CEndorse(void)
{
	m_pCProperty = NULL;
	m_vLogoImage.clear();
}

CEndorse::~CEndorse(void)
{
}

int CEndorse::Init(CProperty_1* pCCProperty)
{
	int nResult = SUCCESS;

	m_pCProperty = pCCProperty;

	GetAPIProcAddress();

	return nResult;
}

int CEndorse::LoadEndorseFile(void)
{
	int nResult = SUCCESS;

	// Set print information for physical endorsement
	nResult = BiSetPrintStation(m_pCProperty->GetHandle(), MF_ST_PHYSICAL_ENDORSEMENT);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Compose path
	TCHAR szPath[_MAX_PATH] = {0};
	::GetModuleFileName(NULL, szPath, _countof(szPath));
	*( ::PathFindFileName(szPath) ) = _T('\0');

	CString strIniPath = CString(szPath) + _T("FormatSettingStep4.ini");
	CString strBmpPath = CString(szPath) + _T("Logo.bmp");

	// Loading printing region setting file for template printing
	int nLoadNum = 0;
	nResult = BiLoadTemplatePrintArea(m_pCProperty->GetHandle(), strIniPath.GetBuffer(), &nLoadNum);
	strIniPath.ReleaseBuffer();
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set print information for electronic endorsement
	nResult = BiSetPrintStation(m_pCProperty->GetHandle(), MF_ST_E_ENDORSEMENT);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	nResult = BiLoadTemplatePrintArea(m_pCProperty->GetHandle(), strIniPath.GetBuffer(), &nLoadNum);
	strIniPath.ReleaseBuffer();
	if(nResult != SUCCESS)
	{
		return nResult;
	}
	
	// Loading image file for printing
	HANDLE hFile = ::CreateFile(strBmpPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile == INVALID_HANDLE_VALUE)
	{
		return ERR_IMAGE_FILEOPEN;
	}

	DWORD dwImageSize = ::GetFileSize(hFile, NULL);
	DWORD dwReadSize = 0;
	m_vLogoImage.resize(dwImageSize, 0);
	BOOL bResult = ::ReadFile(hFile, &m_vLogoImage[0], dwImageSize, &dwReadSize, NULL);
	::CloseHandle(hFile);

	if ( bResult == FALSE )
	{
		return ERR_IMAGE_FILEREAD;
	}

	return nResult;
}

int CEndorse::SetEndorseStation(void)
{
	int nResult = SUCCESS;

	// Select print station
	nResult = BiSetPrintStation(m_pCProperty->GetHandle(), m_pCProperty->GetEndorseType());

	return nResult;
}

int CEndorse::PhysicalEndorseFunction(unsigned long ulTransactionNumber)
{
	int nResult = SUCCESS;

	// Start buffering of print data for format printing
	nResult = BiTemplatePrint(m_pCProperty->GetHandle(), TEMPLATEPRINT_BUFFERING);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set image data for printing
	if(m_pCProperty->GetPrintImage())
	{
		nResult = BiSetTemplatePrintArea(m_pCProperty->GetHandle(), SELECTPRINTAREA_AREANAME, "PhysicalEndorseImage", NULL, ulTransactionNumber);
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		nResult = BiSCNPrintMemoryImage(m_pCProperty->GetHandle(), ulTransactionNumber, &m_vLogoImage[0], static_cast<DWORD>(m_vLogoImage.size()));
		if(nResult != SUCCESS)
		{
			return nResult;
		}
	}

	// set strings for printing
	if(m_pCProperty->GetPrintText())
	{
		MF_DECORATE tDecorate;
		tDecorate.dwAttribute = MF_PRINT_NO_ATTRIBUTE;
		tDecorate.wFont = MF_PRINT_SYSTEMFONT;
		tDecorate.szFontName = "Arial";
		tDecorate.wFontSize = 9;

		char *pcEndorseString;
		CString strEndorseString = NULL;
		CString strAddString = NULL;
		SYSTEMTIME stSystemTime;
		char *pcMonth[] = {"","Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

		/*PhysicalEndorseText1*/
		nResult = BiSetTemplatePrintArea(m_pCProperty->GetHandle(), SELECTPRINTAREA_AREANAME, "PhysicalEndorseText1", NULL, ulTransactionNumber);
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		GetSystemTime(&stSystemTime);

		strAddString.Format( _T("  %s %02d %04d\n"), pcMonth[stSystemTime.wMonth], stSystemTime.wDay,stSystemTime.wYear );
		strEndorseString += strAddString;

		strAddString.Format( _T("  %02d:%02d:%02d:%d\n\n"),
			stSystemTime.wHour, stSystemTime.wMinute, stSystemTime.wSecond, stSystemTime.wMilliseconds );
		strEndorseString += strAddString;
		
		strAddString.Format( _T("  &>1234567890&<") );
		strEndorseString += strAddString;

		pcEndorseString = new char[strEndorseString.GetLength() + 1];
		strcpy_s( pcEndorseString, strEndorseString.GetLength() + 1, strEndorseString );

		nResult = BiSCNPrintText(m_pCProperty->GetHandle(), ulTransactionNumber, pcEndorseString, &tDecorate);
		delete[] pcEndorseString;
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		/*PhysicalEndorseText2*/
		nResult = BiSetTemplatePrintArea(m_pCProperty->GetHandle(), SELECTPRINTAREA_AREANAME, "PhysicalEndorseText2", NULL, ulTransactionNumber);
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		pcEndorseString = "  &>1234567890&<";
		nResult = BiSCNPrintText(m_pCProperty->GetHandle(), ulTransactionNumber, pcEndorseString, &tDecorate);
		if(nResult != SUCCESS)
		{
			return nResult;
		}
	}

	// Print buffered data
	nResult = BiTemplatePrint(m_pCProperty->GetHandle(), TEMPLATEPRINT_EXEC);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

int CEndorse::ElectronicEndorseFunction(void)
{
	int nResult = SUCCESS;

	// Start buffering of print data for format printing
	nResult = BiTemplatePrint(m_pCProperty->GetHandle(), TEMPLATEPRINT_BUFFERING);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	// Set image data for printing
	if(m_pCProperty->GetPrintImage())
	{
		/*ElectronicEndorseImage*/
		nResult = BiSetTemplatePrintArea(m_pCProperty->GetHandle(), SELECTPRINTAREA_AREANAME, "ElectronicEndorseImage", NULL, TRANS_NO_SPECIFIED);
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		// Compose path, StatusAPI supports only MBCS.
		char szPath[_MAX_PATH] = {0};
		::GetModuleFileNameA(NULL, szPath, _countof(szPath));
		*( ::PathFindFileNameA(szPath) ) = _T('\0');
		::strcat_s( szPath, _countof(szPath), "Logo.bmp" );

		nResult = BiPrintImage(m_pCProperty->GetHandle(), szPath);
		if(nResult != SUCCESS)
		{
			return nResult;
		}
	}

	// set strings for printing
	if(m_pCProperty->GetPrintText())
	{
		MF_DECORATE tDecorate;
		tDecorate.dwAttribute = MF_PRINT_NO_ATTRIBUTE;
		tDecorate.wFont = MF_PRINT_SYSTEMFONT;
		tDecorate.szFontName = "Arial";
		tDecorate.wFontSize = 9;

		char *pcEndorseString;
		CString strEndorseString = NULL;
		CString strAddString = NULL;
		SYSTEMTIME stSystemTime;
		char *pcMonth[] = {"","Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

		/*EndorseText1*/
		nResult = BiSetTemplatePrintArea(m_pCProperty->GetHandle(), SELECTPRINTAREA_AREANAME, "ElectronicEndorseText1", NULL, TRANS_NO_SPECIFIED);
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		GetSystemTime(&stSystemTime);

		strAddString.Format( _T("  %s %02d %04d\n"), pcMonth[stSystemTime.wMonth], stSystemTime.wDay,stSystemTime.wYear );
		strEndorseString += strAddString;

		strAddString.Format( _T("  %02d:%02d:%02d:%d\n\n"),
			stSystemTime.wHour, stSystemTime.wMinute, stSystemTime.wSecond, stSystemTime.wMilliseconds );
		strEndorseString += strAddString;
		
		strAddString.Format( _T("  &>1234567890&<") );
		strEndorseString += strAddString;

		pcEndorseString = new char[strEndorseString.GetLength() + 1];
		strcpy_s( pcEndorseString, strEndorseString.GetLength() + 1, strEndorseString );

		nResult = BiPrintText(m_pCProperty->GetHandle(), pcEndorseString, tDecorate);
		delete[] pcEndorseString;
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		/*EndorseText2*/
		nResult = BiSetTemplatePrintArea(m_pCProperty->GetHandle(), SELECTPRINTAREA_AREANAME, "ElectronicEndorseText2", NULL, TRANS_NO_SPECIFIED);
		if(nResult != SUCCESS)
		{
			return nResult;
		}

		pcEndorseString = "  &>1234567890&<";
		nResult = BiPrintText(m_pCProperty->GetHandle(), pcEndorseString, tDecorate);
		if(nResult != SUCCESS)
		{
			return nResult;
		}
	}

	// Print buffered data
	nResult = BiTemplatePrint(m_pCProperty->GetHandle(), TEMPLATEPRINT_EXEC);
	if(nResult != SUCCESS)
	{
		return nResult;
	}

	return nResult;
}

// Get functions of the API
void CEndorse::GetAPIProcAddress(void)
{
	HINSTANCE hLibrary = m_pCProperty->GetLibrary();

	BiSetPrintStation			= (DLL_BiSetPrintStation)		GetProcAddress(hLibrary, "BiSetPrintStation");
	BiPrintText					= (DLL_BiPrintText)				GetProcAddress(hLibrary, "BiPrintText");
	BiPrintImage				= (DLL_BiPrintImage)			GetProcAddress(hLibrary, "BiPrintImage");
	BiSCNPrintText				= (DLL_BiSCNPrintText)			GetProcAddress(hLibrary, "BiSCNPrintText");
	BiSCNPrintMemoryImage		= (DLL_BiSCNPrintMemoryImage)	GetProcAddress(hLibrary, "BiSCNPrintMemoryImage");
	BiLoadTemplatePrintArea		= (DLL_BiLoadTemplatePrintArea)	GetProcAddress(hLibrary, "BiLoadTemplatePrintArea");
	BiTemplatePrint				= (DLL_BiTemplatePrint)			GetProcAddress(hLibrary, "BiTemplatePrint");
	BiSetTemplatePrintArea		= (DLL_BiSetTemplatePrintArea)	GetProcAddress(hLibrary, "BiSetTemplatePrintArea");
}
