Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Linq
Imports System.Text
Imports System.Windows.Forms
Imports System.IO
Imports com.epson.bank.driver

Namespace TM_S9000Sample
	Public Partial Class TMS9000SampleForm
		Inherits Form
		Private m_objDriverControl As CApp = Nothing
		Private m_objConfigData As StructByStep = Nothing

		Private Delegate Sub CallbackProcDisplayMicrText(errResult As ErrorCode, objMicrResult As CMicrResult)
		Private Delegate Sub CallbackProcSaveMicrText(errResult As ErrorCode, objMicrResult As CMicrResult)
		Private Delegate Sub CallbackProcDisplayImage(errResult As ErrorCode, objImageResult As CImageResult, bFourSheetScan As Boolean)
		Private Delegate Sub CallbackProcSaveImageData(errResult As ErrorCode, objImageResult As CImageResult)

		Private CallbackFuncDisplayMicrText As CallbackProcDisplayMicrText = Nothing
		Private CallbackFuncSaveMicrText As CallbackProcSaveMicrText = Nothing
		Private CallbackFuncDisplayImageData As CallbackProcDisplayImage = Nothing
		Private CallbackFuncSaveImageData As CallbackProcSaveImageData = Nothing

		Private CallbackFuncProcessError As CApp.CallbackProcProcessError = Nothing
		Private CallbackFuncImage As CApp.CallbackProcImage = Nothing
		Private CallbackFuncMicr As CApp.CallbackProcMicr = Nothing
		Private CallbackFuncPhysicalEndorse As CApp.CallbackProcPhysicalEndorse = Nothing
		Private CallbackFuncElectronicEndorse As CApp.CallbackProcElectronicEndorse = Nothing

		Private m_bScanCancelError As Boolean = False

		Private m_biImage2Front As Bitmap = Nothing
		Private m_biImage2Back As Bitmap = Nothing
		Private m_biImage4RGBFront As Bitmap = Nothing
		Private m_biImage4RGBBack As Bitmap = Nothing
		Private m_biImage4IrFront As Bitmap = Nothing
		Private m_biImage4IrBack As Bitmap = Nothing

		Public Sub New()
			InitializeComponent()
		End Sub

		Protected Overrides Sub Dispose(disposing As Boolean)
			If disposing Then
				If components IsNot Nothing Then
					components.Dispose()
				End If
				' end process
				Dim errRet As ErrorCode = ErrorCode.SUCCESS

				Try
					errRet = TerminateDriver()
					If errRet <> ErrorCode.SUCCESS Then
						ShowErrorMessage(errRet)
					End If
				Catch
					MyBase.Dispose(disposing)
				End Try
			End If
			MyBase.Dispose(disposing)
		End Sub

		Private Sub TMS9000SampleForm_Load(sender As Object, e As EventArgs)
			m_objDriverControl = New CApp()
			m_objConfigData = New StructByStep()

			If InitializeDriver() <> True Then
				Me.Close()
				Return
			End If

			' Initialize data
			InitializeConfigData()

			' Initialize controls
			LoadResourceString()
			UpdateControls()
		End Sub

		'''//////////////////////////////////////////////////////////////
		' Procedure of DriveControl Callback
		'''//////////////////////////////////////////////////////////////

		'''/////////////
		'''/ Step 1
		'''/////////////
		Private Sub CallbackProcProcessError(errResult As ErrorCode)
			If m_objDriverControl Is Nothing Then
				Return
			End If

			Dim eStatus As New ASB()
			' Display message box correspondent to error
			Select Case errResult
				Case ErrorCode.ERR_COVER_OPEN
					Do
						MessageBox.Show(ConstComStr.MSG_01_001, ConstComStr.CAPTION_01_001, MessageBoxButtons.OK, MessageBoxIcon.[Error])

						m_objDriverControl.GetDeviceStatus(eStatus)
					Loop While (eStatus And ASB.ASB_COVER_OPEN) = ASB.ASB_COVER_OPEN
					m_objDriverControl.CancelError()
					Exit Select

				Case ErrorCode.ERR_PAPER_JAM
					If m_objConfigData.nRadio_ScanningMedia = ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER Then
						Do
							MessageBox.Show(ConstComStr.MSG_02_000, ConstComStr.CAPTION_02_000, MessageBoxButtons.OK, MessageBoxIcon.[Error])

							m_objDriverControl.GetDeviceStatus(eStatus)
						Loop While ((eStatus And ASB.ASB_EJECT_SENSOR_NO_PAPER) <> ASB.ASB_EJECT_SENSOR_NO_PAPER) OrElse ((eStatus And ASB.ASB_SLIP_PAPER_SIZE) <> ASB.ASB_SLIP_PAPER_SIZE) OrElse ((eStatus And ASB.ASB_PAPER_INTERMEDIATE) <> ASB.ASB_PAPER_INTERMEDIATE)
						m_objDriverControl.CancelError()
					Else
						MessageBox.Show(ConstComStr.MSG_02_000, ConstComStr.CAPTION_02_000, MessageBoxButtons.OK, MessageBoxIcon.[Error])
						m_objDriverControl.CancelError()
					End If
					Exit Select

				Case ErrorCode.ERR_MICR_BADDATA, ErrorCode.ERR_MICR_NODATA
					Exit Select
				Case Else

					ShowErrorMessage(errResult)
					Exit Select
			End Select
		End Sub

		'''/////////////
		'''/ Step 2
		'''/////////////
		Private Sub CallbackProcImage()
			If m_objDriverControl Is Nothing Then
				Return
			End If
			' Configure display data for each scanning side and light source
			If m_objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGB Then
				' Configure 2 plane data for RGB image
				UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE, False)
				UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE, False)

				' Save image if necessary
				If m_objConfigData.bCheck_Scan_SaveFile = ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON Then
					SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE)
					SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE)
				End If
			ElseIf m_objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_IR Then
				' Configure 2 plane data for IR image
				UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED, False)
				UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED, False)

				' Save image if necessary
				If m_objConfigData.bCheck_Scan_SaveFile = ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON Then
					SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED)
					SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED)
				End If
			ElseIf m_objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR Then
				' Configure 4 plane image
				UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE, True)
				UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE, True)
				UpdateImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED, True)
				UpdateImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED, True)

				' Save image if necessary
				If m_objConfigData.bCheck_Scan_SaveFile = ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON Then
					SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_VISIBLE)
					SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_VISIBLE)
					SaveImageData(ScanSide.MF_SCAN_FACE_FRONT, ImageType.MF_SCAN_IMAGE_INFRARED)
					SaveImageData(ScanSide.MF_SCAN_FACE_BACK, ImageType.MF_SCAN_IMAGE_INFRARED)
				End If
			End If
		End Sub

		'''/////////////
		'''/ Step 3
		'''/////////////
		Private Sub CallbackProcMicr()
			' Obtain MICR data
			Dim cMicrResult As New CMicrResult()
			Dim errResult As ErrorCode = m_objDriverControl.GetMicr(cMicrResult)
			CallbackFuncDisplayMicrText(errResult, cMicrResult)

			' Save MICR data if necessary
			If m_objConfigData.bCheck_MICR_SaveFile = ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON Then
				CallbackSaveMicrText(errResult, cMicrResult)
			End If
		End Sub

		Private Sub CallbackDisplayMicrText(errResult As ErrorCode, cMicrResult As CMicrResult)
			If Me.InvokeRequired Then
				Me.BeginInvoke(CallbackFuncDisplayMicrText, New Object() {errResult, cMicrResult})
			Else
				Dim strMicrRet As String = Nothing

				If errResult = ErrorCode.SUCCESS Then
					edtMicrText.Text = cMicrResult.GetMicrStr()
				Else
					strMicrRet = GetErrorString(errResult)
					edtMicrText.Text = strMicrRet
				End If
			End If
		End Sub

		'''/////////////
		'''/ Step 4
		'''/////////////
		Private Sub CallbackProcPhysicalEndorse(unTransactionNumber As UInteger)
			Dim errResult As ErrorCode = ErrorCode.SUCCESS

			' Specify endorsement text
			If m_objDriverControl IsNot Nothing Then
				errResult = m_objDriverControl.PhysicalEndorseFunction(unTransactionNumber)
				If errResult <> ErrorCode.SUCCESS Then
					ShowErrorMessage(errResult)
				End If
			End If
		End Sub

		Private Sub CallbackProcElectronicEndorse()
			Dim errResult As ErrorCode = ErrorCode.SUCCESS

			' Specify endorsement text
			If m_objDriverControl IsNot Nothing Then
				errResult = m_objDriverControl.ElectronicEndorseFunction()
				If errResult <> ErrorCode.SUCCESS Then
					ShowErrorMessage(errResult)
				End If
			End If
		End Sub

		'''//////////////////////////////////////////////////////////////
		' CTMS9000SampleDlg message handlers
		'''//////////////////////////////////////////////////////////////
		Private Sub stImage2Front_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs)
			If stImage2Front.Image IsNot Nothing Then
				stImage2Front.Image.Dispose()
			End If

			If m_biImage2Front IsNot Nothing Then
				DrawImage(stImage2Front, m_biImage2Front)
			End If
		End Sub

		Private Sub stImage2Back_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs)
			If stImage2Back.Image IsNot Nothing Then
				stImage2Back.Image.Dispose()
			End If

			If m_biImage2Back IsNot Nothing Then
				DrawImage(stImage2Back, m_biImage2Back)
			End If
		End Sub

		Private Sub stImage4RGBFront_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs)
			If stImage4RGBFront.Image IsNot Nothing Then
				stImage4RGBFront.Image.Dispose()
			End If

			If m_biImage4RGBFront IsNot Nothing Then
				DrawImage(stImage4RGBFront, m_biImage4RGBFront)
			End If
		End Sub

		Private Sub stImage4RGBBack_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs)
			If stImage4RGBBack.Image IsNot Nothing Then
				stImage4RGBBack.Image.Dispose()
			End If

			If m_biImage4RGBBack IsNot Nothing Then
				DrawImage(stImage4RGBBack, m_biImage4RGBBack)
			End If
		End Sub

		Private Sub stImage4IRFront_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs)
			If stImage4IRFront.Image IsNot Nothing Then
				stImage4IRFront.Image.Dispose()
			End If

			If m_biImage4IrFront IsNot Nothing Then
				DrawImage(stImage4IRFront, m_biImage4IrFront)
			End If
		End Sub

		Private Sub stImage4IRBack_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs)
			If stImage4IRBack.Image IsNot Nothing Then
				stImage4IRBack.Image.Dispose()
			End If

			If m_biImage4IrBack IsNot Nothing Then
				DrawImage(stImage4IRBack, m_biImage4IrBack)
			End If
		End Sub

		'''//////////////////////////////////////////////////////////////
		' Button Event
		'''//////////////////////////////////////////////////////////////

		'''/////////////
		'''/ Step 1
		'''/////////////
		Private Sub btnScan_Click(sender As Object, e As EventArgs)
			If m_objDriverControl Is Nothing Then
				Return
			End If

			' Reset controls
			'''/////////////
			'''/ Step 2
			'''/////////////
			ClearImage()
			'''/////////////
			'''/ Step 3
			'''/////////////
			edtMicrText.Text = ""

			' Scanning
			If m_objConfigData.nRadio_ScanningMedia = ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER Then
				Dim errRet As ErrorCode = m_objDriverControl.ScanCheck()
				If errRet <> ErrorCode.SUCCESS Then
					ShowErrorMessage(errRet)
					m_objDriverControl.CancelError()
					Return
				End If
			ElseIf m_objConfigData.nRadio_ScanningMedia = ConstComVal.VAL_CONFIGDLG_RADIO_SM_CARD Then
				Dim errRet As ErrorCode = m_objDriverControl.ScanCard()
				If errRet <> ErrorCode.SUCCESS Then
					ShowErrorMessage(errRet)
					m_objDriverControl.CancelError()
					Return
				End If
			End If
		End Sub

		Private Sub btnExit_Click(sender As Object, e As EventArgs)
			Dim errRet As ErrorCode = ErrorCode.SUCCESS

			errRet = m_objDriverControl.ExitDevice()
			If errRet <> ErrorCode.SUCCESS Then
				ShowErrorMessage(errRet)
			End If
			Close()
		End Sub

		Private Sub btnScanCancel_Click(sender As Object, e As EventArgs)
			If m_objDriverControl Is Nothing Then
				Return
			End If

			Dim errRet As ErrorCode = m_objDriverControl.CancelScan()
			If errRet <> ErrorCode.SUCCESS Then
				m_bScanCancelError = True
				ShowErrorMessage(errRet)
				m_objDriverControl.CancelError()
			End If

		End Sub

		'''/////////////
		'''/ Step 2
		'''/////////////

		Private Sub btnConfig_Click(sender As Object, e As EventArgs)
			Dim cConfigureDlg As New ConfigureForm()

			cConfigureDlg.SetConfigData(m_objConfigData)

			If cConfigureDlg.ShowDialog(Me) = DialogResult.OK Then
				cConfigureDlg.GetConfigData(m_objConfigData)

				'''/////////////
				'''/ Step 2
				'''/////////////
				Dim cScanParam As New CScanParam()
				ConvertConfigDataToScanParam(m_objConfigData, cScanParam)
				If m_objDriverControl IsNot Nothing Then
					m_objDriverControl.SetScanParam(cScanParam)
				End If

				'''/////////////
				'''/ Step 3
				'''/////////////
				Dim cMicrParam As New CMicrParam()
				ConvertConfigDataToMicrParam(m_objConfigData, cMicrParam)
				If m_objDriverControl IsNot Nothing Then
					m_objDriverControl.SetMicrParam(cMicrParam)
				End If

				'''/////////////
				'''/ Step 4
				'''/////////////
				Dim cEndorseParam As New CEndorseParam()
				ConvertConfigDataToEndorseParam(m_objConfigData, cEndorseParam)
				If m_objDriverControl IsNot Nothing Then
					m_objDriverControl.SetEndorseParam(cEndorseParam)
				End If

				Dim cOptionParam As New COptionParam()
				ConvertConfigDataToOptionParam(m_objConfigData, cOptionParam)
				If m_objDriverControl IsNot Nothing Then
					m_objDriverControl.SetOptionParam(cOptionParam)
				End If

				UpdateControls()
			End If
		End Sub

		'''/////////////
		'''/ Step 3
		'''/////////////
		Private Sub btnMicrCleaning_Click(sender As Object, e As EventArgs)
			If m_objDriverControl Is Nothing Then
				Return
			End If

			Dim errRet As ErrorCode = m_objDriverControl.CleaningMicr()
			If errRet <> ErrorCode.SUCCESS Then
				ShowErrorMessage(errRet)
				m_objDriverControl.CancelError()
			End If
		End Sub

		'''//////////////////////////////////////////////////////////////
		' Member functions
		'''//////////////////////////////////////////////////////////////

		Private Function InitializeDriver() As Boolean
			Dim errResult As ErrorCode = ErrorCode.SUCCESS
			Dim drRet As DialogResult = 0
			Dim strResult As String = Nothing

			If m_objDriverControl Is Nothing Then
				Return False
			End If

			' Open device and register callback functions
			errResult = m_objDriverControl.InitDevice()
			While errResult <> ErrorCode.SUCCESS
				If errResult = ErrorCode.ERR_UNKNOWN Then
					drRet = MessageBox.Show(ConstComStr.MSG_06_000, ConstComStr.CAPTION_06_000, MessageBoxButtons.OKCancel, MessageBoxIcon.[Error])
				Else
					strResult = GetErrorString(errResult)
					drRet = MessageBox.Show(ConstComStr.MSG_00_000, strResult, MessageBoxButtons.OKCancel, MessageBoxIcon.[Error])
				End If

				If drRet = DialogResult.OK Then
					errResult = m_objDriverControl.InitDevice()
				Else
					Return False
				End If
			End While

			errResult = m_objDriverControl.LoadEndorseFile()
			Select Case errResult
				Case ErrorCode.SUCCESS
					Exit Select

				Case ErrorCode.ERR_NOT_FOUND
					MessageBox.Show(ConstComStr.MSG_00_001, ConstComStr.CAPTION_03_042, MessageBoxButtons.OK, MessageBoxIcon.[Error])
					Return False

				Case ErrorCode.ERR_IMAGE_FILEOPEN
					MessageBox.Show(ConstComStr.MSG_00_002, ConstComStr.CAPTION_03_039, MessageBoxButtons.OK, MessageBoxIcon.[Error])
					Return False

				Case ErrorCode.ERR_IMAGE_FILEREAD
					MessageBox.Show(ConstComStr.MSG_00_002, ConstComStr.CAPTION_03_049, MessageBoxButtons.OK, MessageBoxIcon.[Error])
					Return False
				Case Else

					ShowErrorMessage(errResult)
					Return False
			End Select

			CallbackFuncDisplayMicrText = New CallbackProcDisplayMicrText(AddressOf CallbackDisplayMicrText)
			CallbackFuncSaveMicrText = New CallbackProcSaveMicrText(AddressOf CallbackSaveMicrText)
			CallbackFuncDisplayImageData = New CallbackProcDisplayImage(AddressOf CallbackDisplayImageData)
			CallbackFuncSaveImageData = New CallbackProcSaveImageData(AddressOf CallbackSaveImageData)

			CallbackFuncProcessError = New CApp.CallbackProcProcessError(AddressOf CallbackProcProcessError)
			CallbackFuncImage = New CApp.CallbackProcImage(AddressOf CallbackProcImage)
			CallbackFuncMicr = New CApp.CallbackProcMicr(AddressOf CallbackProcMicr)
			CallbackFuncPhysicalEndorse = New CApp.CallbackProcPhysicalEndorse(AddressOf CallbackProcPhysicalEndorse)
			CallbackFuncElectronicEndorse = New CApp.CallbackProcElectronicEndorse(AddressOf CallbackProcElectronicEndorse)

			' Specify callback function from DriverControl
			m_objDriverControl.SetProcessErrorCallback(CallbackFuncProcessError)
			m_objDriverControl.SetImageCallback(CallbackFuncImage)
			m_objDriverControl.SetMicrCallback(CallbackFuncMicr)
			'Step4
			m_objDriverControl.SetPhysicalEndorseCallback(CallbackFuncPhysicalEndorse)
			m_objDriverControl.SetElectronicEndorseCallback(CallbackFuncElectronicEndorse)

			Return True
		End Function

		Private Function TerminateDriver() As ErrorCode
			Dim errRet As ErrorCode = ErrorCode.SUCCESS

			If m_objDriverControl Is Nothing Then
				Return ErrorCode.ERR_NO_MEMORY
			End If

			errRet = m_objDriverControl.ExitDevice()
			If errRet <> ErrorCode.SUCCESS Then
				Return errRet
			End If

			Return errRet
		End Function

		Private Sub ShowErrorMessage(errResult As ErrorCode)
			Select Case errResult
				Case ErrorCode.ERR_TYPE
					MessageBox.Show(ConstComStr.MSG_03_000, ConstComStr.CAPTION_03_000, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_OPENED
					MessageBox.Show(ConstComStr.MSG_03_001, ConstComStr.CAPTION_03_001, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_NO_PRINTER
					MessageBox.Show(ConstComStr.MSG_03_002, ConstComStr.CAPTION_03_002, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_NO_TARGET
					MessageBox.Show(ConstComStr.MSG_03_003, ConstComStr.CAPTION_03_003, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_NO_MEMORY
					MessageBox.Show(ConstComStr.MSG_03_004, ConstComStr.CAPTION_03_004, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_HANDLE
					MessageBox.Show(ConstComStr.MSG_03_005, ConstComStr.CAPTION_03_005, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_TIMEOUT
					MessageBox.Show(ConstComStr.MSG_03_006, ConstComStr.CAPTION_03_006, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_ACCESS
					MessageBox.Show(ConstComStr.MSG_03_007, ConstComStr.CAPTION_03_007, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PARAM
					MessageBox.Show(ConstComStr.MSG_03_008, ConstComStr.CAPTION_03_008, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_NOT_SUPPORT
					MessageBox.Show(ConstComStr.MSG_03_009, ConstComStr.CAPTION_03_009, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_OFFLINE
					MessageBox.Show(ConstComStr.MSG_03_010, ConstComStr.CAPTION_03_010, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_NOT_EPSON
					MessageBox.Show(ConstComStr.MSG_03_011, ConstComStr.CAPTION_03_011, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_WITHOUT_CB
					If m_bScanCancelError = True Then
						MessageBox.Show(ConstComStr.MSG_03_012_01, ConstComStr.CAPTION_03_012, MessageBoxButtons.OK, MessageBoxIcon.Warning)
						m_bScanCancelError = False
					Else
						MessageBox.Show(ConstComStr.MSG_03_012_00, ConstComStr.CAPTION_03_012, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					End If
					Exit Select
				Case ErrorCode.ERR_BUFFER_OVER_FLOW
					MessageBox.Show(ConstComStr.MSG_03_013, ConstComStr.CAPTION_03_013, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_REGISTRY
					MessageBox.Show(ConstComStr.MSG_03_014, ConstComStr.CAPTION_03_014, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PAPERINSERT_TIMEOUT
					MessageBox.Show(ConstComStr.MSG_03_015, ConstComStr.CAPTION_03_015, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_FUNCTION
					MessageBox.Show(ConstComStr.MSG_03_016, ConstComStr.CAPTION_03_016, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_MICR
					MessageBox.Show(ConstComStr.MSG_03_017, ConstComStr.CAPTION_03_017, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_SCAN
					MessageBox.Show(ConstComStr.MSG_03_018, ConstComStr.CAPTION_03_018, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_RESET
					MessageBox.Show(ConstComStr.MSG_03_019, ConstComStr.CAPTION_03_019, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_ABORT
					MessageBox.Show(ConstComStr.MSG_03_020, ConstComStr.CAPTION_03_020, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_MICR
					MessageBox.Show(ConstComStr.MSG_03_021, ConstComStr.CAPTION_03_021, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_SCAN
					MessageBox.Show(ConstComStr.MSG_03_022, ConstComStr.CAPTION_03_022, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_LINE_OVERFLOW
					MessageBox.Show(ConstComStr.MSG_03_023, ConstComStr.CAPTION_03_023, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PAPER_PILED
					MessageBox.Show(ConstComStr.MSG_03_024, ConstComStr.CAPTION_03_024, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PAPER_JAM
					MessageBox.Show(ConstComStr.MSG_03_025, ConstComStr.CAPTION_03_025, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_COVER_OPEN
					MessageBox.Show(ConstComStr.MSG_03_026, ConstComStr.CAPTION_03_026, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_MICR_NODATA
					MessageBox.Show(ConstComStr.MSG_03_027, ConstComStr.CAPTION_03_027, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_MICR_BADDATA
					MessageBox.Show(ConstComStr.MSG_03_028, ConstComStr.CAPTION_03_028, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_MICR_NOISE
					MessageBox.Show(ConstComStr.MSG_03_029, ConstComStr.CAPTION_03_029, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_SCN_COMPRESS
					MessageBox.Show(ConstComStr.MSG_03_030, ConstComStr.CAPTION_03_030, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PAPER_EXIST
					MessageBox.Show(ConstComStr.MSG_03_031, ConstComStr.CAPTION_03_031, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PAPER_INSERT
					MessageBox.Show(ConstComStr.MSG_03_032, ConstComStr.CAPTION_03_032, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_SCAN_CHECK_CONTINUOUS
					MessageBox.Show(ConstComStr.MSG_03_033, ConstComStr.CAPTION_03_033, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_SCAN_CHECK_ONEBYONE
					MessageBox.Show(ConstComStr.MSG_03_034, ConstComStr.CAPTION_03_034, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_SCAN_IDCARD
					MessageBox.Show(ConstComStr.MSG_03_035, ConstComStr.CAPTION_03_035, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_PRINT_ROLLPAPER
					MessageBox.Show(ConstComStr.MSG_03_036, ConstComStr.CAPTION_03_036, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_EXEC_PRINT_VALIDATION
					MessageBox.Show(ConstComStr.MSG_03_037, ConstComStr.CAPTION_03_037, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_THREAD
					MessageBox.Show(ConstComStr.MSG_03_038, ConstComStr.CAPTION_03_038, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_IMAGE_FILEOPEN
					MessageBox.Show(ConstComStr.MSG_03_039, ConstComStr.CAPTION_03_039, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_IMAGE_UNKNOWNFORMAT
					MessageBox.Show(ConstComStr.MSG_03_040, ConstComStr.CAPTION_03_040, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_SIZE
					MessageBox.Show(ConstComStr.MSG_03_041, ConstComStr.CAPTION_03_041, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_NOT_FOUND
					MessageBox.Show(ConstComStr.MSG_03_042, ConstComStr.CAPTION_03_042, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_NOT_EXEC
					MessageBox.Show(ConstComStr.MSG_03_043, ConstComStr.CAPTION_03_043, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_BARCODE_NODATA
					MessageBox.Show(ConstComStr.MSG_03_044, ConstComStr.CAPTION_03_044, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_MICR_PARSE
					MessageBox.Show(ConstComStr.MSG_03_045, ConstComStr.CAPTION_03_045, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_SCN_IQA
					MessageBox.Show(ConstComStr.MSG_03_046, ConstComStr.CAPTION_03_046, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PRINT_DATA_LENGTH_EXCEED
					MessageBox.Show(ConstComStr.MSG_03_047, ConstComStr.CAPTION_03_047, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_PRINT_DATA_UNRECEIVE
					MessageBox.Show(ConstComStr.MSG_03_048, ConstComStr.CAPTION_03_048, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case ErrorCode.ERR_IMAGE_FILEREAD
					MessageBox.Show(ConstComStr.MSG_03_049, ConstComStr.CAPTION_03_049, MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
				Case Else
					MessageBox.Show(errResult.ToString(), "ERR", MessageBoxButtons.OK, MessageBoxIcon.Warning)
					Exit Select
			End Select
		End Sub

		Private Function GetErrorString(errResultCode As ErrorCode) As String
			Select Case errResultCode

				Case ErrorCode.SUCCESS
					Return ConstComStr.CAPTION_00_000
				Case ErrorCode.ERR_TYPE
					Return ConstComStr.CAPTION_03_000
				Case ErrorCode.ERR_OPENED
					Return ConstComStr.CAPTION_03_001
				Case ErrorCode.ERR_NO_PRINTER
					Return ConstComStr.CAPTION_03_002
				Case ErrorCode.ERR_NO_TARGET
					Return ConstComStr.CAPTION_03_003
				Case ErrorCode.ERR_NO_MEMORY
					Return ConstComStr.CAPTION_03_004
				Case ErrorCode.ERR_HANDLE
					Return ConstComStr.CAPTION_03_005
				Case ErrorCode.ERR_TIMEOUT
					Return ConstComStr.CAPTION_03_006
				Case ErrorCode.ERR_ACCESS
					Return ConstComStr.CAPTION_03_007
				Case ErrorCode.ERR_PARAM
					Return ConstComStr.CAPTION_03_008
				Case ErrorCode.ERR_NOT_SUPPORT
					Return ConstComStr.CAPTION_03_009
				Case ErrorCode.ERR_OFFLINE
					Return ConstComStr.CAPTION_03_010
				Case ErrorCode.ERR_NOT_EPSON
					Return ConstComStr.CAPTION_03_011
				Case ErrorCode.ERR_WITHOUT_CB
					Return ConstComStr.CAPTION_03_012
				Case ErrorCode.ERR_BUFFER_OVER_FLOW
					Return ConstComStr.CAPTION_03_013
				Case ErrorCode.ERR_REGISTRY
					Return ConstComStr.CAPTION_03_014
				Case ErrorCode.ERR_PAPERINSERT_TIMEOUT
					Return ConstComStr.CAPTION_03_015
				Case ErrorCode.ERR_EXEC_FUNCTION
					Return ConstComStr.CAPTION_03_016
				Case ErrorCode.ERR_EXEC_MICR
					Return ConstComStr.CAPTION_03_017
				Case ErrorCode.ERR_EXEC_SCAN
					Return ConstComStr.CAPTION_03_018
				Case ErrorCode.ERR_RESET
					Return ConstComStr.CAPTION_03_019
				Case ErrorCode.ERR_ABORT
					Return ConstComStr.CAPTION_03_020
				Case ErrorCode.ERR_MICR
					Return ConstComStr.CAPTION_03_021
				Case ErrorCode.ERR_SCAN
					Return ConstComStr.CAPTION_03_022
				Case ErrorCode.ERR_LINE_OVERFLOW
					Return ConstComStr.CAPTION_03_023
				Case ErrorCode.ERR_PAPER_PILED
					Return ConstComStr.CAPTION_03_024
				Case ErrorCode.ERR_PAPER_JAM
					Return ConstComStr.CAPTION_03_025
				Case ErrorCode.ERR_COVER_OPEN
					Return ConstComStr.CAPTION_03_026
				Case ErrorCode.ERR_MICR_NODATA
					Return ConstComStr.CAPTION_03_027
				Case ErrorCode.ERR_MICR_BADDATA
					Return ConstComStr.CAPTION_03_028
				Case ErrorCode.ERR_MICR_NOISE
					Return ConstComStr.CAPTION_03_029
				Case ErrorCode.ERR_SCN_COMPRESS
					Return ConstComStr.CAPTION_03_030
				Case ErrorCode.ERR_PAPER_EXIST
					Return ConstComStr.CAPTION_03_031
				Case ErrorCode.ERR_PAPER_INSERT
					Return ConstComStr.CAPTION_03_032
				Case ErrorCode.ERR_EXEC_SCAN_CHECK_CONTINUOUS
					Return ConstComStr.CAPTION_03_033
				Case ErrorCode.ERR_EXEC_SCAN_CHECK_ONEBYONE
					Return ConstComStr.CAPTION_03_034
				Case ErrorCode.ERR_EXEC_SCAN_IDCARD
					Return ConstComStr.CAPTION_03_035
				Case ErrorCode.ERR_EXEC_PRINT_ROLLPAPER
					Return ConstComStr.CAPTION_03_036
				Case ErrorCode.ERR_EXEC_PRINT_VALIDATION
					Return ConstComStr.CAPTION_03_037
				Case ErrorCode.ERR_THREAD
					Return ConstComStr.CAPTION_03_038
				Case ErrorCode.ERR_IMAGE_FILEOPEN
					Return ConstComStr.CAPTION_03_039
				Case ErrorCode.ERR_IMAGE_UNKNOWNFORMAT
					Return ConstComStr.CAPTION_03_040
				Case ErrorCode.ERR_SIZE
					Return ConstComStr.CAPTION_03_041
				Case ErrorCode.ERR_NOT_FOUND
					Return ConstComStr.CAPTION_03_042
				Case ErrorCode.ERR_NOT_EXEC
					Return ConstComStr.CAPTION_03_043
				Case ErrorCode.ERR_BARCODE_NODATA
					Return ConstComStr.CAPTION_03_044
				Case ErrorCode.ERR_MICR_PARSE
					Return ConstComStr.CAPTION_03_045
				Case ErrorCode.ERR_SCN_IQA
					Return ConstComStr.CAPTION_03_046
				Case ErrorCode.ERR_PRINT_DATA_LENGTH_EXCEED
					Return ConstComStr.CAPTION_03_047
				Case ErrorCode.ERR_PRINT_DATA_UNRECEIVE
					Return ConstComStr.CAPTION_03_048
				Case ErrorCode.ERR_IMAGE_FILEREAD
					Return ConstComStr.CAPTION_03_049
				Case Else
					Return errResultCode.ToString()
			End Select
		End Function

		Private Sub InitializeConfigData()
			If m_objConfigData Is Nothing Then
				Return
			End If

			'''/////////////
			'''/ Step 2 ////
			'''/////////////

			Dim cScanParam As New CScanParam()
			m_objDriverControl.GetScanParam(cScanParam)
			ConvertScanParamToConfigData(cScanParam, m_objConfigData)

			'''/////////////
			'''/ Step 3 ////
			'''/////////////

			Dim cMicrParam As New CMicrParam()
			m_objDriverControl.GetMicrParam(cMicrParam)
			ConvertMicrParamToConfigData(cMicrParam, m_objConfigData)

			'''/////////////
			'''/ Step 4 ////
			'''/////////////

			Dim cEndorseParam As New CEndorseParam()
			m_objDriverControl.GetEndorseParam(cEndorseParam)
			ConvertEndorseParamToConfigData(cEndorseParam, m_objConfigData)

			Dim cOptionParam As New COptionParam()
			m_objDriverControl.GetOptionParam(cOptionParam)
			ConvertOptionParamToConfigData(cOptionParam, m_objConfigData)
		End Sub

		Private Sub LoadResourceString()
			'''/////////////
			'''/ Step 1 ////
			'''/////////////

			' Title
			Me.Text = ConstComStr.STR_SAMPLEDLG_TITLE

			' Scan Button
			btnScan.Text = ConstComStr.STR_SAMPLEDLG_BUTTON_SCAN

			' ScanCancel Button
			btnScanCancel.Text = ConstComStr.STR_SAMPLEDLG_BUTTON_SCANCANCEL

			' Exit Button
			btnExit.Text = ConstComStr.STR_SAMPLEDLG_BUTTON_EXIT


			'''/////////////
			'''/ Step 2 ////
			'''/////////////

			' Config Button
			btnConfig.Text = ConstComStr.STR_SAMPLEDLG_BUTTON_CONFIG


			'''/////////////
			'''/ Step 3 ////
			'''/////////////

			' MICR Text Static
			stMicrText.Text = ConstComStr.STR_SAMPLEDLG_STATIC_MICRTEXT

			' Micr Cleaning Button
			btnMicrCleaning.Text = ConstComStr.STR_SAMPLEDLG_BUTTON_MICRCLEANING

		End Sub

		Private Sub UpdateControls()
			'''/////////////
			'''/ Step 2 ////
			'''/////////////
			If m_objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR Then
				stImage2Front.Visible = False
				stImage2Back.Visible = False
				stImage4RGBFront.Visible = True
				stImage4RGBBack.Visible = True
				stImage4IRBack.Visible = True
				stImage4IRFront.Visible = True
			Else
				stImage2Front.Visible = True
				stImage2Back.Visible = True
				stImage4RGBFront.Visible = False
				stImage4RGBBack.Visible = False
				stImage4IRBack.Visible = False
				stImage4IRFront.Visible = False
			End If
		End Sub

		Private Sub ConvertConfigDataToScanParam(ByRef objConfigData As StructByStep, ByRef objScanParam As CScanParam)
			If objScanParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objConfigData.nRadio_ScanningMedia
				Case ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER
					objScanParam.SetScanMedia(ScanUnit.EPS_BI_SCN_UNIT_CHECKPAPER)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_SM_CARD
					objScanParam.SetScanMedia(ScanUnit.EPS_BI_SCN_UNIT_CARD)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.nRadio_ImageChannel
				' ScanGradation
				Case ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGB
					If objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR Then
						objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_COLOR)
					Else
						objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE)
					End If
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_IC_IR
					objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_IR)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR
					If objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR Then
						objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_COLOR_IR)
					Else
						objScanParam.SetImageChannel(ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE_IR)
					End If
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.nRadio_RGBColorDepth
				' ScanGradation
				Case ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR
					objScanParam.SetRGBColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_24BIT)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_GRAY
					objScanParam.SetRGBColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_BW
					objScanParam.SetRGBColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.nRadio_IRColorDepth
				' ScanGradation
				Case ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_GRAY
					objScanParam.SetIRColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_BW
					objScanParam.SetIRColorDepth(com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.nCombo_Resolution
				' ScanGradation
				Case ConstComVal.VAL_CONFIGDLG_COMBO_RE_600600
					objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_600)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_COMBO_RE_300300
					objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_300)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_COMBO_RE_240240
					objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_240)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_COMBO_RE_200200
					objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_200)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_COMBO_RE_120120
					objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_120)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_COMBO_RE_100100
					objScanParam.SetResolution(MfScanDpi.MF_SCAN_DPI_100)
					Exit Select
				Case Else
					Exit Select
			End Select
		End Sub

		Private Sub ConvertConfigDataToMicrParam(ByRef objConfigData As StructByStep, ByRef objMicrParam As CMicrParam)
			If objMicrParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objConfigData.nRadio_MICRFont
				Case ConstComVal.VAL_CONFIGDLG_RADIO_MF_E13B
					objMicrParam.SetFont(MfMicrFont.MF_MICR_FONT_E13B)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_MF_CMC7
					objMicrParam.SetFont(MfMicrFont.MF_MICR_FONT_CMC7)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.bCheck_MICR_ClearSpace
				Case ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_ON
					objMicrParam.SetClearSpace(True)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_OFF
					objMicrParam.SetClearSpace(False)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.bCheck_MICR_SaveFile
				Case ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_ON
					objMicrParam.SetParsing(True)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_CHECK_MI_SAVEFILE_OFF
					objMicrParam.SetParsing(False)
					Exit Select
				Case Else
					Exit Select
			End Select
		End Sub

		'''/////////////
		'''/ Step 4
		'''/////////////
		Private Sub ConvertConfigDataToEndorseParam(ByRef objConfigData As StructByStep, ByRef objEndorseParam As CEndorseParam)
			If objEndorseParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objConfigData.nRadio_PrintType
				Case ConstComVal.VAL_CONFIGDLG_RADIO_PT_PHYSICAL
					objEndorseParam.SetEndorseType(PrintingStation.MF_ST_PHYSICAL_ENDORSEMENT)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_PT_ELECTRONIC
					objEndorseParam.SetEndorseType(PrintingStation.MF_ST_E_ENDORSEMENT)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.bCheck_Endorsement_PrintText
				' ScanGradation
				Case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_ON
					objEndorseParam.SetPrintText(True)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_OFF
					objEndorseParam.SetPrintText(False)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.bCheck_Endorsement_PrintImage
				' ScanGradation
				Case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_ON
					objEndorseParam.SetPrintImage(True)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_OFF
					objEndorseParam.SetPrintImage(False)
					Exit Select
				Case Else
					Exit Select
			End Select
		End Sub

		Private Sub ConvertConfigDataToOptionParam(ByRef objConfigData As StructByStep, ByRef objOptionParam As COptionParam)
			If objOptionParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objConfigData.nRadio_PrintMode
				Case ConstComVal.VAL_CONFIGDLG_RADIO_PM_HIGHSPEEDMODE
					objOptionParam.SetEndorsePrintMode(MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_HIGHSPEED)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_RADIO_PM_DATAWAITINGMODE
					objOptionParam.SetEndorsePrintMode(MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_DATAWAITING)
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objConfigData.bCheck_Endorsement_PrnDataUnreceiveCancel
				Case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_ON
					objOptionParam.SetPrnDataUnreceiveCancel(MfCancel.MF_CANCEL_ENABLE)
					Exit Select
				Case ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_OFF
					objOptionParam.SetPrnDataUnreceiveCancel(MfCancel.MF_CANCEL_DISABLE)
					Exit Select
				Case Else
					Exit Select
			End Select
		End Sub

		Private Sub ConvertScanParamToConfigData(ByRef objScanParam As CScanParam, ByRef objConfigData As StructByStep)
			If objScanParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objScanParam.GetScanMedia()
				Case ScanUnit.EPS_BI_SCN_UNIT_CHECKPAPER
					objConfigData.nRadio_ScanningMedia = ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER
					Exit Select
				Case ScanUnit.EPS_BI_SCN_UNIT_CARD
					objConfigData.nRadio_ScanningMedia = ConstComVal.VAL_CONFIGDLG_RADIO_SM_CARD
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objScanParam.GetImageChannel()
				' Scan Gradation
				Case ImageTypeOption.EPS_BI_SCN_OPTION_COLOR, ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE, ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_RED, ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_GREEN, ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_BLUE
					objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGB
					Exit Select
				Case ImageTypeOption.EPS_BI_SCN_OPTION_IR
					objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_IR
					Exit Select
				Case ImageTypeOption.EPS_BI_SCN_OPTION_COLOR_IR, ImageTypeOption.EPS_BI_SCN_OPTION_GRAYSCALE_IR, ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_RED_IR, ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_GREEN_IR, ImageTypeOption.EPS_BI_SCN_OPTION_DROPOUT_BLUE_IR
					objConfigData.nRadio_ImageChannel = ConstComVal.VAL_CONFIGDLG_RADIO_IC_RGBIR
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objScanParam.GetRGBColorDepth()
				' RGB Color Depth
				Case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_24BIT
					objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_COLOR
					Exit Select
				Case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT
					objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_GRAY
					Exit Select
				Case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT
					objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_BW
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objScanParam.GetIRColorDepth()
				' IR Color Depth
				Case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_8BIT
					objConfigData.nRadio_IRColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_GRAY
					Exit Select
				Case com.epson.bank.driver.ColorDepth.EPS_BI_SCN_1BIT
					objConfigData.nRadio_IRColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_BW
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objScanParam.GetResolution()
				' Resoluton
				Case MfScanDpi.MF_SCAN_DPI_600
					objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_600600
					Exit Select
				Case MfScanDpi.MF_SCAN_DPI_300
					objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_300300
					Exit Select
				Case MfScanDpi.MF_SCAN_DPI_240
					objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_240240
					Exit Select
				Case MfScanDpi.MF_SCAN_DPI_200
					objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_200200
					Exit Select
				Case MfScanDpi.MF_SCAN_DPI_120
					objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_120120
					Exit Select
				Case MfScanDpi.MF_SCAN_DPI_100
					objConfigData.nCombo_Resolution = ConstComVal.VAL_CONFIGDLG_COMBO_RE_100100
					Exit Select
				Case Else
					Exit Select
			End Select
		End Sub

		Private Sub ConvertMicrParamToConfigData(ByRef objMicrParam As CMicrParam, ByRef objConfigData As StructByStep)
			If objMicrParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objMicrParam.GetFont()
				Case MfMicrFont.MF_MICR_FONT_E13B
					objConfigData.nRadio_MICRFont = ConstComVal.VAL_CONFIGDLG_RADIO_MF_E13B
					Exit Select
				Case MfMicrFont.MF_MICR_FONT_CMC7
					objConfigData.nRadio_MICRFont = ConstComVal.VAL_CONFIGDLG_RADIO_MF_CMC7
					Exit Select
				Case Else
					Exit Select
			End Select

			If objMicrParam.GetClearSpace() = True Then
				objConfigData.bCheck_MICR_ClearSpace = ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_ON
			Else
				objConfigData.bCheck_MICR_ClearSpace = ConstComVal.VAL_CONFIGDLG_CHECK_MI_CLEARSPACE_OFF
			End If
		End Sub

		'''/////////////
		'''/ Step 4
		'''/////////////
		Private Sub ConvertEndorseParamToConfigData(ByRef objEndorseParam As CEndorseParam, ByRef objConfigData As StructByStep)

			If objEndorseParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objEndorseParam.GetEndorseType()
				' Endorse Type
				Case PrintingStation.MF_ST_PHYSICAL_ENDORSEMENT
					objConfigData.nRadio_PrintType = ConstComVal.VAL_CONFIGDLG_RADIO_PT_PHYSICAL
					Exit Select
				Case PrintingStation.MF_ST_E_ENDORSEMENT
					objConfigData.nRadio_PrintType = ConstComVal.VAL_CONFIGDLG_RADIO_PT_ELECTRONIC
					Exit Select
				Case Else
					Exit Select
			End Select

			If objEndorseParam.GetPrintText() = True Then
				objConfigData.bCheck_Endorsement_PrintText = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_ON
			Else
				objConfigData.bCheck_Endorsement_PrintText = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTTEXT_OFF
			End If

			If objEndorseParam.GetPrintImage() = True Then
				objConfigData.bCheck_Endorsement_PrintImage = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_ON
			Else
				objConfigData.bCheck_Endorsement_PrintImage = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRINTIMAGE_OFF
			End If
		End Sub

		Private Sub ConvertOptionParamToConfigData(ByRef objOptionParam As COptionParam, ByRef objConfigData As StructByStep)

			If objOptionParam Is Nothing OrElse objConfigData Is Nothing Then
				Return
			End If

			Select Case objOptionParam.GetEndorsePrintMode()
				Case MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_HIGHSPEED
					objConfigData.nRadio_PrintMode = ConstComVal.VAL_CONFIGDLG_RADIO_PM_HIGHSPEEDMODE
					Exit Select
				Case MfEndorsePrintMode.MF_ENDORSEPRINT_MODE_DATAWAITING
					objConfigData.nRadio_PrintMode = ConstComVal.VAL_CONFIGDLG_RADIO_PM_DATAWAITINGMODE
					Exit Select
				Case Else
					Exit Select
			End Select

			Select Case objOptionParam.GetPrnDataUnreceiveCancel()
				Case MfCancel.MF_CANCEL_DISABLE
					objConfigData.bCheck_Endorsement_PrnDataUnreceiveCancel = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_OFF
					Exit Select
				Case MfCancel.MF_CANCEL_ENABLE
					objConfigData.bCheck_Endorsement_PrnDataUnreceiveCancel = ConstComVal.VAL_CONFIGDLG_CHECK_EN_PRNDATAUNRECEIVECANCEL_ON
					Exit Select
				Case Else
					Exit Select
			End Select
		End Sub

		Private Sub UpdateImageData(eFace As ScanSide, eImage As ImageType, bFourSheetScan As Boolean)
			If m_objDriverControl Is Nothing Then
				Return
			End If

			Dim cImageResult As New CImageResult()

			' Obtain image data
			cImageResult.SetFace(eFace)
			cImageResult.SetGradation(eImage)
			cImageResult.SetFormat(Format.EPS_BI_SCN_BITMAP)
			Dim errResult As ErrorCode = ErrorCode.SUCCESS

			errResult = m_objDriverControl.GetScanImage(cImageResult)

			CallbackDisplayImageData(errResult, cImageResult, bFourSheetScan)
		End Sub

		Private Sub CallbackDisplayImageData(errResult As ErrorCode, cImageResult As CImageResult, bFourSheetScan As Boolean)
			If Me.InvokeRequired Then
				Me.BeginInvoke(CallbackFuncDisplayImageData, New Object() {errResult, cImageResult, bFourSheetScan})
			Else
				If errResult <> ErrorCode.SUCCESS Then
					ShowErrorMessage(errResult)
					Return
				End If

				Dim cBitmap As New Bitmap(cImageResult.GetBitmapImage())
				If cBitmap IsNot Nothing Then
					cImageResult.FreeImage()
				End If

				If bFourSheetScan = False Then
					If cImageResult.GetFace() = ScanSide.MF_SCAN_FACE_FRONT Then
						m_biImage2Front = New Bitmap(cBitmap)
					Else
						m_biImage2Back = New Bitmap(cBitmap)
					End If
				Else
					If cImageResult.GetGradation() = ImageType.MF_SCAN_IMAGE_VISIBLE Then
						If cImageResult.GetFace() = ScanSide.MF_SCAN_FACE_FRONT Then
							m_biImage4RGBFront = New Bitmap(cBitmap)
						Else
							m_biImage4RGBBack = New Bitmap(cBitmap)
						End If
					Else
						If cImageResult.GetFace() = ScanSide.MF_SCAN_FACE_FRONT Then
							m_biImage4IrFront = New Bitmap(cBitmap)
						Else
							m_biImage4IrBack = New Bitmap(cBitmap)
						End If
					End If
				End If

				cBitmap.Dispose()
				Me.Refresh()
			End If
		End Sub

		Private Sub SaveImageData(eFace As ScanSide, eImage As ImageType)
			If m_objDriverControl Is Nothing Then
				Return
			End If

			Dim cImageResult As New CImageResult()

			' Specify image data format
			Dim eFormat As Format = Format.EPS_BI_SCN_JPEGNORMAL
			If eImage = ImageType.MF_SCAN_IMAGE_VISIBLE AndAlso m_objConfigData.nRadio_RGBColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_RGBCD_BW Then
				eFormat = Format.EPS_BI_SCN_TIFF
			End If
			If eImage = ImageType.MF_SCAN_IMAGE_INFRARED AndAlso m_objConfigData.nRadio_IRColorDepth = ConstComVal.VAL_CONFIGDLG_RADIO_IRCD_BW Then
				eFormat = Format.EPS_BI_SCN_TIFF
			End If

			' Obtain image data
			cImageResult.SetFace(eFace)
			cImageResult.SetGradation(eImage)
			cImageResult.SetFormat(eFormat)
			Dim errResult As ErrorCode = ErrorCode.SUCCESS

			errResult = m_objDriverControl.GetScanImage(cImageResult)

			CallbackSaveImageData(errResult, cImageResult)
		End Sub

		Private Sub CallbackSaveImageData(errResult As ErrorCode, objImageResult As CImageResult)
			If Me.InvokeRequired Then
				Me.BeginInvoke(CallbackFuncSaveImageData, New Object() {errResult, objImageResult})
			Else
				If errResult <> ErrorCode.SUCCESS Then
					ShowErrorMessage(errResult)
					Return
				End If

				Dim cImage As Stream = objImageResult.GetImageData()
				Dim nImageSize As Integer = objImageResult.GetImageSize()
				Dim nTransactionNumber As Integer = objImageResult.GetTransactionNumber()
				Dim strExt As String

				If objImageResult.GetFormat() = Format.EPS_BI_SCN_TIFF Then
					strExt = "tif"
				Else
					strExt = "jpg"
				End If


				' Create Folder
				Dim sbPath As New StringBuilder()
				Dim szPath As String = Nothing

				sbPath.Append(System.IO.Directory.GetCurrentDirectory())
				sbPath.Append("\ScanResult")
				szPath = sbPath.ToString()

				Try
					System.IO.Directory.CreateDirectory(szPath)
				Catch
					MessageBox.Show(ConstComStr.MSG_05_000, ConstComStr.CAPTION_05_000, MessageBoxButtons.OK, MessageBoxIcon.[Error])
					Return
				End Try


				' Save image to file
				sbPath.Append("\" & nTransactionNumber.ToString("000") & ("_Image_") & (If(objImageResult.GetGradation() = ImageType.MF_SCAN_IMAGE_INFRARED, "IR_", "")) & (If(objImageResult.GetFace() = ScanSide.MF_SCAN_FACE_BACK, "Back", "Front")) & (".") & strExt)
				szPath = sbPath.ToString()

				If objImageResult.GetImageData() IsNot Nothing Then
					Dim cFile As System.IO.FileStream
					Try
						cFile = New System.IO.FileStream(szPath, System.IO.FileMode.Create)
					Catch
						MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_000, MessageBoxButtons.OK, MessageBoxIcon.[Error])
						Return
					End Try

					Dim byBuffer As Byte() = New Byte(objImageResult.GetImageSize() - 1) {}

					' Save in file
					Try
						objImageResult.GetImageData().Read(byBuffer, 0, byBuffer.Length)
						cFile.Write(byBuffer, 0, byBuffer.Length)

						objImageResult.GetImageData().Close()
					Catch
						MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_001, MessageBoxButtons.OK, MessageBoxIcon.[Error])
						Return
					Finally
						cFile.Close()
					End Try
				End If
			End If
		End Sub

		Private Sub ClearImage()
			If stImage2Front.Image IsNot Nothing Then
				stImage2Front.Image.Dispose()
				stImage2Front.Image = Nothing
				m_biImage2Front = Nothing
			End If
			If stImage2Back.Image IsNot Nothing Then
				stImage2Back.Image.Dispose()
				stImage2Back.Image = Nothing
				m_biImage2Back = Nothing
			End If
			If stImage4RGBFront.Image IsNot Nothing Then
				stImage4RGBFront.Image.Dispose()
				stImage4RGBFront.Image = Nothing
				m_biImage4RGBFront = Nothing
			End If
			If stImage4RGBBack.Image IsNot Nothing Then
				stImage4RGBBack.Image.Dispose()
				stImage4RGBBack.Image = Nothing
				m_biImage4RGBBack = Nothing
			End If
			If stImage4IRBack.Image IsNot Nothing Then
				stImage4IRBack.Image.Dispose()
				stImage4IRBack.Image = Nothing
				m_biImage4IrBack = Nothing
			End If
			If stImage4IRFront.Image IsNot Nothing Then
				stImage4IRFront.Image.Dispose()
				stImage4IRFront.Image = Nothing
				m_biImage4IrFront = Nothing
			End If
		End Sub

		Private Sub DrawImage(stImage As PictureBox, biImage As Bitmap)
			Dim nWidth As Integer = 0
			Dim nHeight As Integer = 0

			If (biImage.Width * stImage.Height) < (stImage.Width * biImage.Height) Then
				nWidth = stImage.Height * biImage.Width \ biImage.Height
				stImage.Image = New Bitmap(biImage, nWidth, stImage.Height)
			Else
				nHeight = stImage.Width * biImage.Height \ biImage.Width
				stImage.Image = New Bitmap(biImage, stImage.Width, nHeight)
			End If
		End Sub

		Private Sub CallbackSaveMicrText(errResult As ErrorCode, cMicrResult As CMicrResult)
			If Me.InvokeRequired Then
				Me.BeginInvoke(CallbackFuncSaveMicrText, New Object() {errResult, cMicrResult})
			Else
				' Obtain MICR data
				Dim nTransactionNumber As Integer = cMicrResult.GetTransactionNumber()

				If errResult <> ErrorCode.SUCCESS Then
					ShowErrorMessage(errResult)
					Return
				End If

				' Create Folder
				Dim sbPath As New StringBuilder()
				Dim szPath As String = Nothing

				sbPath.Append(System.IO.Directory.GetCurrentDirectory())
				sbPath.Append("\ScanResult")
				szPath = sbPath.ToString()
				Try
					System.IO.Directory.CreateDirectory(szPath)
				Catch
					MessageBox.Show(ConstComStr.MSG_05_000, ConstComStr.CAPTION_05_000, MessageBoxButtons.OK, MessageBoxIcon.[Error])
					Return
				End Try

				' Save data to file
				sbPath.Append("\" & nTransactionNumber.ToString("000") & ".Micr.txt")
				szPath = sbPath.ToString()

				Dim swFile As System.IO.StreamWriter
				Try
					swFile = New System.IO.StreamWriter(szPath)
				Catch
					MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_001, MessageBoxButtons.OK, MessageBoxIcon.[Error])
					Return
				End Try

				Try
					' Transaction number
					swFile.WriteLine("# General Transaction Information:")
					swFile.WriteLine("TransactionNumber = " & nTransactionNumber.ToString())

					' MICR reading status
					swFile.WriteLine("")
					swFile.WriteLine("# Magnetic Character Results:")
					swFile.WriteLine("Status = " & cMicrResult.GetStatus().ToString())

					' Detail MICR reading status
					swFile.WriteLine("Detail = " & cMicrResult.GetDetail().ToString())

					' Return value from MICR recognition
					swFile.WriteLine("MicrResult = " & GetErrorString(cMicrResult.GetResult()))

					' Obtained MICR strings
					swFile.WriteLine("MicrString = " & (If(cMicrResult.GetMicrStr() IsNot Nothing, cMicrResult.GetMicrStr(), "")))

					'MicrAccountNumber
					swFile.WriteLine("MicrAccountNumber = " & (If(cMicrResult.GetAccountNumber() IsNot Nothing, cMicrResult.GetAccountNumber(), "")))

					'MicrAmount
					swFile.WriteLine("MicrAmount        = " & (If(cMicrResult.GetAmount() IsNot Nothing, cMicrResult.GetAmount(), "")))

					'MicrBankNumber
					swFile.WriteLine("MicrBankNumber    = " & (If(cMicrResult.GetBankNumber() IsNot Nothing, cMicrResult.GetBankNumber(), "")))

					'MicrSerialNumber
					swFile.WriteLine("MicrSerialNumber  = " & (If(cMicrResult.GetSerialNumber() IsNot Nothing, cMicrResult.GetSerialNumber(), "")))

					'MicrEPC
					swFile.WriteLine("MicrEPC           = " & (If(cMicrResult.GetEPC() IsNot Nothing, cMicrResult.GetEPC(), "")))

					' MicrTransitNumber
					swFile.WriteLine("MicrTransitNumber = " & (If(cMicrResult.GetTransitNumber() IsNot Nothing, cMicrResult.GetTransitNumber(), "")))

					' MicrCheckType
					swFile.WriteLine("MicrCheckType     = " & cMicrResult.GetCheckType().ToString())

					' MicrCountryCode
					swFile.WriteLine("MicrCountryCode     = " & cMicrResult.GetCountryCode().ToString())

					' MicrOnUSField
					swFile.WriteLine("MicrOnUSField = " & (If(cMicrResult.GetOnUSField() IsNot Nothing, cMicrResult.GetOnUSField(), "")))

					' MicrAuxiliatyOnUSField

					swFile.WriteLine("MicrAuxiliatyOnUSField = " & (If(cMicrResult.GetAuxiliatyOnUSField() IsNot Nothing, cMicrResult.GetAuxiliatyOnUSField(), "")))
				Catch
					MessageBox.Show(ConstComStr.MSG_05_001, ConstComStr.CAPTION_05_001, MessageBoxButtons.OK, MessageBoxIcon.[Error])
					Return
				Finally
					swFile.Close()
				End Try
			End If
		End Sub
	End Class
End Namespace
